@@ -321,7 +321,7 @@ class CloneMixin(object):
         new_instance = cls()
 
         for f in fields:
-            value = getattr(instance, f.attname, f.get_default())
+            value = f.get_default() or getattr(instance, f.attname, None)
 
             if isinstance(f, (models.DateTimeField, models.DateField)):
                 if f.auto_now or f.auto_now_add:
@@ -407,7 +407,7 @@ class CloneMixin(object):
         :type duplicate: `django.db.models.Model`
         :param using: The database alias used to save the created instances.
         :type using: str
-        :return: The duplicate instance with all the transcient one to many duplicated instances.
+        :return: The duplicate instance with all the transient one to many duplicated instances.
         """
 
         for f in itertools.chain(
